@@ -9,11 +9,14 @@ use Net::SSH::Perl::Config;
 use Net::SSH::Perl::Constants qw( :protocol :compat :hosts );
 use Net::SSH::Perl::Cipher;
 use Net::SSH::Perl::Util qw( :hosts _read_yes_or_no );
+use Data::Dumper;
+
+use Errno qw( EAGAIN EWOULDBLOCK );
 
 use vars qw( $VERSION $CONFIG $HOSTNAME );
 $CONFIG = {};
 
-use Socket;
+use Socket qw(IPPROTO_TCP TCP_NODELAY);
 use IO::Socket;
 use Fcntl;
 use Symbol;
@@ -23,7 +26,7 @@ eval {
     $HOSTNAME = hostname();
 };
 
-$VERSION = '1.35';
+$VERSION = '1.36';
 
 sub VERSION { $VERSION }
 
@@ -232,6 +235,7 @@ sub _create_socket {
     for(; $p != $end; $p += $delta) {
         socket($sock, AF_INET, SOCK_STREAM, getprotobyname('tcp') || 0) ||
             croak "Net::SSH: Can't create socket: $!";
+        setsockopt($sock, IPPROTO_TCP, TCP_NODELAY, 1);
         last if not $p or bind($sock, sockaddr_in($p,$addr));
         if ($! =~ /Address already in use/i) {
             close($sock) or warn qq{Could not close socket: $!\n};
@@ -255,12 +259,44 @@ sub fatal_disconnect {
     croak @_;
 }
 
+sub _read_version_line {
+    my $ssh = shift;
+    my $sock = $ssh->{session}{sock};
+    my $line;
+    for(;;) {
+        my $s = IO::Select->new($sock);
+        my @ready = $s->can_read;
+        my $buf;
+        my $len = sysread($sock, $buf, 1);
+        unless(defined($len)) {
+            next if $! == EAGAIN || $! == EWOULDBLOCK;
+            croak "Read from socket failed: $!";
+        }
+        croak "Connection closed by remote host" if $len == 0;
+        $line .= $buf;
+        croak "Version line too long: $line"
+         if substr($line, 0, 4) eq "SSH-" and length($line) > 255;
+        croak "Pre-version line too long: $line" if length($line) > 4*1024;
+        return $line if $buf eq "\n";
+    }
+}
+
+sub _read_version {
+    my $ssh = shift;
+    my $line;
+    do {
+        $line = $ssh->_read_version_line;
+    } while (substr($line, 0, 4) ne "SSH-");
+    $ssh->debug("Remote version string: $line");
+    return $line;
+}
+
 sub sock { $_[0]->{session}{sock} }
 
 sub _exchange_identification {
     my $ssh = shift;
     my $sock = $ssh->{session}{sock};
-    my $remote_id = <$sock>;
+    my $remote_id = $ssh->_read_version;
     ($ssh->{server_version_string} = $remote_id) =~ s/\cM?\n$//;
     my($remote_major, $remote_minor, $remote_version) = $remote_id =~
         /^SSH-(\d+)\.(\d+)-([^\n]+)\n$/;
@@ -296,7 +332,7 @@ sub _exchange_identification {
         $compat20 ? PROTOCOL_MINOR_2 : PROTOCOL_MINOR_1,
         $VERSION;
     $ssh->{client_version_string} = substr $buf, 0, -1;
-    print $sock $buf;
+    syswrite $sock, $buf;
 
     $ssh->set_protocol($set_proto);
     $ssh->_compat_init($remote_version);
@@ -346,12 +382,14 @@ sub packet_start { Net::SSH::Perl::Packet->new($_[0], type => $_[1]) }
 sub check_host_key {
     my $ssh = shift;
     my($key, $host, $u_hostfile, $s_hostfile) = @_;
+    my $strict_host_key_checking = $ssh->{config}->get('strict_host_key_checking');
+    $strict_host_key_checking ||= 'no';
     $host ||= $ssh->{host};
     $u_hostfile ||= $ssh->{config}->get('user_known_hosts');
     $s_hostfile ||= $ssh->{config}->get('global_known_hosts');
 
     my $status = _check_host_in_hostfile($host, $u_hostfile, $key);
-    unless (defined $status && $status == HOST_OK) {
+    unless (defined $status && ($status == HOST_OK || $status == HOST_CHANGED)) {
         $status = _check_host_in_hostfile($host, $s_hostfile, $key);
     }
 
@@ -359,7 +397,10 @@ sub check_host_key {
         $ssh->debug("Host '$host' is known and matches the host key.");
     }
     elsif ($status == HOST_NEW) {
-        if ($ssh->{config}->get('interactive')) {
+        if ($strict_host_key_checking =~ /(ask|yes)/) {
+            if (!$ssh->{config}->get('interactive')) {
+                croak "Host key verification failed.";
+            }
             my $prompt =
 qq(The authenticity of host '$host' can't be established.
 Key fingerprint is @{[ $key->fingerprint ]}.
@@ -587,6 +628,27 @@ you probably won't need to use this, often.
 
 The default is 1 if you're starting up a shell, and 0 otherwise.
 
+=item * terminal_mode_string
+
+Specify the POSIX terminal mode string to send when use_pty is
+set. By default the only mode set is the VEOF character to 0x04
+(opcode 5, value 0x00000004). See RFC 4254 section 8 for complete
+details on this value.
+
+=item * no_append_veof
+
+(SSH-2 only) Set this to 1 if you specified use_pty and do not want
+Ctrl-D (0x04) appended twice to the end of your input string. On most
+systems, these bytes cause the terminal driver to return "EOF" when
+standard input is read. Without them, many programs that read from
+standard input will hang after consuming all the data on STDIN.
+
+No other modifications are made to input data. If your data contains
+0x04 bytes, you may need to escape them.
+
+Set this to 0 if you have raw terminal data to specify on standard
+input, and you have terminated it correctly.
+
 =item * options
 
 Used to specify additional options to the configuration settings;
